/*
 * Copyright (c) 2012 Eric Haszlakiewicz
 * Copyright (c) 2016 Adiscon GmbH
 * Rainer Gerhards <rgerhards@adiscon.com>
 *
 * This library is free software; you can redistribute it and/or modify
 * it under the terms of the MIT license. See COPYING for details.
 */
#include "config.h"
#include "json.h"

const char *fjson_version(void)
{
	return VERSION;
}
